@@ -18,6 +18,7 @@ cd build
 # Run with options not normally exercised by the rpm build, for
 # more complete code coverage.
 ../autogen.sh --prefix="$AUTOBUILD_INSTALL_ROOT" \
+  --enable-expensive-tests \
   --enable-test-coverage \
   --disable-nls \
   --enable-werror \
@@ -69,12 +70,14 @@ fi
 if test -x /usr/bin/i686-w64-mingw32-gcc ; then
   make distclean
 
+  PKG_CONFIG_LIBDIR="/usr/i686-w64-mingw32/sys-root/mingw/lib/pkgconfig:/usr/i686-w64-mingw32/sys-root/mingw/share/pkgconfig" \
   PKG_CONFIG_PATH="$AUTOBUILD_INSTALL_ROOT/i686-w64-mingw32/sys-root/mingw/lib/pkgconfig" \
   CC="i686-w64-mingw32-gcc" \
   ../configure \
     --build=$(uname -m)-w64-linux \
     --host=i686-w64-mingw32 \
     --prefix="$AUTOBUILD_INSTALL_ROOT/i686-w64-mingw32/sys-root/mingw" \
+    --enable-expensive-tests \
     --enable-werror \
     --without-libvirtd \
     --without-python
@@ -88,12 +91,14 @@ fi
 if test -x /usr/bin/x86_64-w64-mingw32-gcc ; then
   make distclean
 
+  PKG_CONFIG_LIBDIR="/usr/x86_64-w64-mingw32/sys-root/mingw/lib/pkgconfig:/usr/x86_64-w64-mingw32/sys-root/mingw/share/pkgconfig" \
   PKG_CONFIG_PATH="$AUTOBUILD_INSTALL_ROOT/x86_64-w64-mingw32/sys-root/mingw/lib/pkgconfig" \
   CC="x86_64-w64-mingw32-gcc" \
   ../configure \
     --build=$(uname -m)-w64-linux \
     --host=x86_64-w64-mingw32 \
     --prefix="$AUTOBUILD_INSTALL_ROOT/x86_64-w64-mingw32/sys-root/mingw" \
+    --enable-expensive-tests \
     --enable-werror \
     --without-libvirtd \
     --without-python
