@@ -263,7 +263,7 @@ daemonPidFilePath(bool privileged,
 
         if (virAsprintf(pidfile, "%s/libvirtd.pid", rundir) < 0) {
             VIR_FREE(rundir);
-            goto no_memory;
+            goto error;
         }
 
         VIR_FREE(rundir);
@@ -271,8 +271,6 @@ daemonPidFilePath(bool privileged,
 
     return 0;
 
-no_memory:
-    virReportOOMError();
 error:
     return -1;
 }
@@ -285,10 +283,10 @@ daemonUnixSocketPaths(struct daemonConfig *config,
 {
     if (config->unix_sock_dir) {
         if (virAsprintf(sockfile, "%s/libvirt-sock", config->unix_sock_dir) < 0)
-            goto no_memory;
+            goto error;
         if (privileged &&
             virAsprintf(rosockfile, "%s/libvirt-sock-ro", config->unix_sock_dir) < 0)
-            goto no_memory;
+            goto error;
     } else {
         if (privileged) {
             if (VIR_STRDUP(*sockfile, LOCALSTATEDIR "/run/libvirt/libvirt-sock") < 0 ||
@@ -310,7 +308,7 @@ daemonUnixSocketPaths(struct daemonConfig *config,
 
             if (virAsprintf(sockfile, "%s/libvirt-sock", rundir) < 0) {
                 VIR_FREE(rundir);
-                goto no_memory;
+                goto error;
             }
 
             VIR_FREE(rundir);
@@ -318,8 +316,6 @@ daemonUnixSocketPaths(struct daemonConfig *config,
     }
     return 0;
 
-no_memory:
-    virReportOOMError();
 error:
     return -1;
 }
@@ -489,6 +485,7 @@ static int daemonSetupNetworking(virNetServerPtr srv,
                                            NULL,
 #endif
                                            false,
+                                           config->max_queued_clients,
                                            config->max_client_requests)))
         goto error;
     if (sock_path_ro) {
@@ -501,6 +498,7 @@ static int daemonSetupNetworking(virNetServerPtr srv,
                                                  NULL,
 #endif
                                                  true,
+                                                 config->max_queued_clients,
                                                  config->max_client_requests)))
             goto error;
     }
@@ -526,6 +524,7 @@ static int daemonSetupNetworking(virNetServerPtr srv,
                                                      NULL,
 #endif
                                                      false,
+                                                     config->max_queued_clients,
                                                      config->max_client_requests)))
                 goto error;
 
@@ -566,6 +565,7 @@ static int daemonSetupNetworking(virNetServerPtr srv,
                                             config->auth_tls,
                                             ctxt,
                                             false,
+                                            config->max_queued_clients,
                                             config->max_client_requests))) {
                 virObjectUnref(ctxt);
                 goto error;
@@ -666,7 +666,7 @@ daemonSetupLogging(struct daemonConfig *config,
         char *tmp;
         if (access("/run/systemd/journal/socket", W_OK) >= 0) {
             if (virAsprintf(&tmp, "%d:journald", virLogGetDefaultPriority()) < 0)
-                goto no_memory;
+                goto error;
             virLogParseOutputs(tmp);
             VIR_FREE(tmp);
         }
@@ -684,7 +684,7 @@ daemonSetupLogging(struct daemonConfig *config,
                 if (virAsprintf(&tmp, "%d:file:%s/log/libvirt/libvirtd.log",
                                 virLogGetDefaultPriority(),
                                 LOCALSTATEDIR) == -1)
-                    goto no_memory;
+                    goto error;
             } else {
                 char *logdir = virGetUserCacheDirectory();
                 mode_t old_umask;
@@ -702,13 +702,13 @@ daemonSetupLogging(struct daemonConfig *config,
                 if (virAsprintf(&tmp, "%d:file:%s/libvirtd.log",
                                 virLogGetDefaultPriority(), logdir) == -1) {
                     VIR_FREE(logdir);
-                    goto no_memory;
+                    goto error;
                 }
                 VIR_FREE(logdir);
             }
         } else {
             if (virAsprintf(&tmp, "%d:stderr", virLogGetDefaultPriority()) < 0)
-                goto no_memory;
+                goto error;
         }
         virLogParseOutputs(tmp);
         VIR_FREE(tmp);
@@ -722,8 +722,6 @@ daemonSetupLogging(struct daemonConfig *config,
 
     return 0;
 
-no_memory:
-    virReportOOMError();
 error:
     return -1;
 }
@@ -1040,12 +1038,13 @@ daemonUsage(const char *argv0, bool privileged)
               "  %s [options]\n"
               "\n"
               "Options:\n"
+              "  -h | --help            Display program help:\n"
               "  -v | --verbose         Verbose messages.\n"
               "  -d | --daemon          Run as a daemon & write PID file.\n"
               "  -l | --listen          Listen for TCP/IP connections.\n"
               "  -t | --timeout <secs>  Exit after timeout period.\n"
               "  -f | --config <file>   Configuration file.\n"
-              "     | --version         Display version information.\n"
+              "  -V | --version         Display version information.\n"
               "  -p | --pid-file <file> Change name of PID file.\n"
               "\n"
               "libvirt management daemon:\n"),
@@ -1100,10 +1099,6 @@ daemonUsage(const char *argv0, bool privileged)
     }
 }
 
-enum {
-    OPT_VERSION = 129
-};
-
 #define MAX_LISTEN 5
 int main(int argc, char **argv) {
     virNetServerPtr srv = NULL;
@@ -1125,14 +1120,14 @@ int main(int argc, char **argv) {
     mode_t old_umask;
 
     struct option opts[] = {
-        { "verbose", no_argument, &verbose, 1},
-        { "daemon", no_argument, &godaemon, 1},
-        { "listen", no_argument, &ipsock, 1},
+        { "verbose", no_argument, &verbose, 'v'},
+        { "daemon", no_argument, &godaemon, 'd'},
+        { "listen", no_argument, &ipsock, 'l'},
         { "config", required_argument, NULL, 'f'},
         { "timeout", required_argument, NULL, 't'},
         { "pid-file", required_argument, NULL, 'p'},
-        { "version", no_argument, NULL, OPT_VERSION },
-        { "help", no_argument, NULL, '?' },
+        { "version", no_argument, NULL, 'V' },
+        { "help", no_argument, NULL, 'h' },
         {0, 0, 0, 0}
     };
 
@@ -1153,7 +1148,7 @@ int main(int argc, char **argv) {
         }
         *tmp = '\0';
         char *driverdir;
-        if (virAsprintf(&driverdir, "%s/../../src/.libs", argv[0]) < 0) {
+        if (virAsprintfQuiet(&driverdir, "%s/../../src/.libs", argv[0]) < 0) {
             fprintf(stderr, _("%s: initialization failed\n"), argv[0]);
             exit(EXIT_FAILURE);
         }
@@ -1175,7 +1170,7 @@ int main(int argc, char **argv) {
         int c;
         char *tmp;
 
-        c = getopt_long(argc, argv, "ldf:p:t:v", opts, &optidx);
+        c = getopt_long(argc, argv, "ldf:p:t:vVh", opts, &optidx);
 
         if (c == -1) {
             break;
@@ -1221,17 +1216,17 @@ int main(int argc, char **argv) {
             }
             break;
 
-        case OPT_VERSION:
+        case 'V':
             daemonVersion(argv[0]);
-            return 0;
+            exit(EXIT_SUCCESS);
+
+        case 'h':
+            daemonUsage(argv[0], privileged);
+            exit(EXIT_SUCCESS);
 
         case '?':
-            daemonUsage(argv[0], privileged);
-            return 2;
-
         default:
-            VIR_ERROR(_("%s: internal error: unknown flag: %c"),
-                      argv[0], c);
+            daemonUsage(argv[0], privileged);
             exit(EXIT_FAILURE);
         }
     }
