@@ -119,6 +119,7 @@ typedef enum {
     VIR_FROM_CGROUP = 54,       /* Error from cgroups */
 
     VIR_FROM_ACCESS = 55,       /* Error from access control manager */
+    VIR_FROM_SYSTEMD = 56,      /* Error from systemd code */
 
 # ifdef VIR_ENUM_SENTINELS
     VIR_ERR_DOMAIN_LAST
@@ -294,6 +295,7 @@ typedef enum {
     VIR_ERR_RESOURCE_BUSY = 87,         /* resource is already in use */
     VIR_ERR_ACCESS_DENIED = 88,         /* operation on the object/resource
                                            was denied */
+    VIR_ERR_DBUS_SERVICE = 89,          /* error from a dbus service */
 } virErrorNumber;
 
 /**
