@@ -115,6 +115,8 @@ struct _virNetServer {
 
 static virClassPtr virNetServerClass;
 static void virNetServerDispose(void *obj);
+static void virNetServerUpdateServicesLocked(virNetServerPtr srv,
+                                             bool enabled);
 
 static int virNetServerOnceInit(void)
 {
@@ -221,10 +223,8 @@ static int virNetServerDispatchNewMessage(virNetServerClientPtr client,
     if (srv->workers) {
         virNetServerJobPtr job;
 
-        if (VIR_ALLOC(job) < 0) {
-            virReportOOMError();
+        if (VIR_ALLOC(job) < 0)
             goto cleanup;
-        }
 
         job->client = client;
         job->msg = msg;
@@ -267,13 +267,17 @@ static int virNetServerAddClient(virNetServerPtr srv,
     if (virNetServerClientInit(client) < 0)
         goto error;
 
-    if (VIR_EXPAND_N(srv->clients, srv->nclients, 1) < 0) {
-        virReportOOMError();
+    if (VIR_EXPAND_N(srv->clients, srv->nclients, 1) < 0)
         goto error;
-    }
     srv->clients[srv->nclients-1] = client;
     virObjectRef(client);
 
+    if (srv->nclients == srv->nclients_max) {
+        /* Temporarily stop accepting new clients */
+        VIR_DEBUG("Temporarily suspending services due to max_clients");
+        virNetServerUpdateServicesLocked(srv, false);
+    }
+
     virNetServerClientSetDispatcher(client,
                                     virNetServerDispatchNewMessage,
                                     srv);
@@ -856,7 +860,7 @@ virNetServerSignalEvent(int watch,
                         void *opaque) {
     virNetServerPtr srv = opaque;
     siginfo_t siginfo;
-    int i;
+    size_t i;
 
     virObjectLock(srv);
 
@@ -933,10 +937,10 @@ int virNetServerAddSignalHandler(virNetServerPtr srv,
         goto error;
 
     if (VIR_EXPAND_N(srv->signals, srv->nsignals, 1) < 0)
-        goto no_memory;
+        goto error;
 
     if (VIR_ALLOC(sigdata) < 0)
-        goto no_memory;
+        goto error;
 
     sigdata->signum = signum;
     sigdata->func = func;
@@ -954,8 +958,6 @@ int virNetServerAddSignalHandler(virNetServerPtr srv,
     virObjectUnlock(srv);
     return 0;
 
-no_memory:
-    virReportOOMError();
 error:
     VIR_FREE(sigdata);
     virObjectUnlock(srv);
@@ -971,7 +973,7 @@ int virNetServerAddService(virNetServerPtr srv,
     virObjectLock(srv);
 
     if (VIR_EXPAND_N(srv->services, srv->nservices, 1) < 0)
-        goto no_memory;
+        goto error;
 
     if (mdnsEntryName) {
         int port = virNetServerServiceGetPort(svc);
@@ -992,8 +994,6 @@ int virNetServerAddService(virNetServerPtr srv,
     virObjectUnlock(srv);
     return 0;
 
-no_memory:
-    virReportOOMError();
 error:
     virObjectUnlock(srv);
     return -1;
@@ -1005,15 +1005,14 @@ int virNetServerAddProgram(virNetServerPtr srv,
     virObjectLock(srv);
 
     if (VIR_EXPAND_N(srv->programs, srv->nprograms, 1) < 0)
-        goto no_memory;
+        goto error;
 
     srv->programs[srv->nprograms-1] = virObjectRef(prog);
 
     virObjectUnlock(srv);
     return 0;
 
-no_memory:
-    virReportOOMError();
+error:
     virObjectUnlock(srv);
     return -1;
 }
@@ -1043,15 +1042,22 @@ static void virNetServerAutoShutdownTimer(int timerid ATTRIBUTE_UNUSED,
 }
 
 
+static void
+virNetServerUpdateServicesLocked(virNetServerPtr srv,
+                                 bool enabled)
+{
+    size_t i;
+
+    for (i = 0; i < srv->nservices; i++)
+        virNetServerServiceToggle(srv->services[i], enabled);
+}
+
+
 void virNetServerUpdateServices(virNetServerPtr srv,
                                 bool enabled)
 {
-    int i;
-
     virObjectLock(srv);
-    for (i = 0; i < srv->nservices; i++)
-        virNetServerServiceToggle(srv->services[i], enabled);
-
+    virNetServerUpdateServicesLocked(srv, enabled);
     virObjectUnlock(srv);
 }
 
@@ -1060,7 +1066,7 @@ void virNetServerRun(virNetServerPtr srv)
 {
     int timerid = -1;
     bool timerActive = false;
-    int i;
+    size_t i;
 
     virObjectLock(srv);
 
@@ -1129,6 +1135,14 @@ void virNetServerRun(virNetServerPtr srv)
                     srv->nclients = 0;
                 }
 
+                /* Enable services if we can accept a new client.
+                 * The new client can be accepted if we are at the limit. */
+                if (srv->nclients == srv->nclients_max - 1) {
+                    /* Now it makes sense to accept() a new client. */
+                    VIR_DEBUG("Re-enabling services");
+                    virNetServerUpdateServicesLocked(srv, true);
+                }
+
                 virObjectUnlock(srv);
                 virObjectUnref(client);
                 virObjectLock(srv);
@@ -1156,7 +1170,7 @@ void virNetServerQuit(virNetServerPtr srv)
 void virNetServerDispose(void *obj)
 {
     virNetServerPtr srv = obj;
-    int i;
+    size_t i;
 
     VIR_FORCE_CLOSE(srv->autoShutdownInhibitFd);
 
@@ -1195,7 +1209,7 @@ void virNetServerDispose(void *obj)
 
 void virNetServerClose(virNetServerPtr srv)
 {
-    int i;
+    size_t i;
 
     if (!srv)
         return;
