@@ -315,10 +315,8 @@ virNetDevReplaceMacAddress(const char *linkdev,
 
     if (virAsprintf(&path, "%s/%s",
                     stateDir,
-                    linkdev) < 0) {
-        virReportOOMError();
+                    linkdev) < 0)
         return -1;
-    }
     virMacAddrFormat(&oldmac, macstr);
     if (virFileWriteStr(path, macstr, O_CREAT|O_TRUNC|O_WRONLY) < 0) {
         virReportSystemError(errno, _("Unable to preserve mac for %s"),
@@ -352,10 +350,8 @@ virNetDevRestoreMacAddress(const char *linkdev,
 
     if (virAsprintf(&path, "%s/%s",
                     stateDir,
-                    linkdev) < 0) {
-        virReportOOMError();
+                    linkdev) < 0)
         return -1;
-    }
 
     if (virFileReadAll(path, VIR_MAC_STRING_BUFLEN, &macstr) < 0)
         return -1;
@@ -505,10 +501,8 @@ int virNetDevSetNamespace(const char *ifname, pid_t pidInNs)
         "ip", "link", "set", ifname, "netns", NULL, NULL
     };
 
-    if (virAsprintf(&pid, "%lld", (long long) pidInNs) == -1) {
-        virReportOOMError();
+    if (virAsprintf(&pid, "%lld", (long long) pidInNs) == -1)
         return -1;
-    }
 
     argv[5] = pid;
     rc = virRun(argv, NULL);
@@ -816,12 +810,25 @@ int virNetDevSetIPv4Address(const char *ifname,
          !(bcaststr = virSocketAddrFormat(&broadcast)))) {
         goto cleanup;
     }
+#ifdef IFCONFIG_PATH
+    cmd = virCommandNew(IFCONFIG_PATH);
+    virCommandAddArg(cmd, ifname);
+    if (VIR_SOCKET_ADDR_IS_FAMILY(addr, AF_INET6))
+        virCommandAddArg(cmd, "inet6");
+    else
+        virCommandAddArg(cmd, "inet");
+    virCommandAddArgFormat(cmd, "%s/%u", addrstr, prefix);
+    if (bcaststr)
+        virCommandAddArgList(cmd, "broadcast", bcaststr, NULL);
+    virCommandAddArg(cmd, "alias");
+#else
     cmd = virCommandNew(IP_PATH);
     virCommandAddArgList(cmd, "addr", "add", NULL);
     virCommandAddArgFormat(cmd, "%s/%u", addrstr, prefix);
     if (bcaststr)
         virCommandAddArgList(cmd, "broadcast", bcaststr, NULL);
     virCommandAddArgList(cmd, "dev", ifname, NULL);
+#endif
 
     if (virCommandRun(cmd, NULL) < 0)
         goto cleanup;
@@ -901,10 +908,21 @@ int virNetDevClearIPv4Address(const char *ifname,
 
     if (!(addrstr = virSocketAddrFormat(addr)))
         goto cleanup;
+#ifdef IFCONFIG_PATH
+    cmd = virCommandNew(IFCONFIG_PATH);
+    virCommandAddArg(cmd, ifname);
+    if (VIR_SOCKET_ADDR_IS_FAMILY(addr, AF_INET6))
+        virCommandAddArg(cmd, "inet6");
+    else
+        virCommandAddArg(cmd, "inet");
+    virCommandAddArgFormat(cmd, "%s/%u", addrstr, prefix);
+    virCommandAddArg(cmd, "-alias");
+#else
     cmd = virCommandNew(IP_PATH);
     virCommandAddArgList(cmd, "addr", "del", NULL);
     virCommandAddArgFormat(cmd, "%s/%u", addrstr, prefix);
     virCommandAddArgList(cmd, "dev", ifname, NULL);
+#endif
 
     if (virCommandRun(cmd, NULL) < 0)
         goto cleanup;
@@ -1056,12 +1074,8 @@ virNetDevSysfsFile(char **pf_sysfs_device_link, const char *ifname,
                const char *file)
 {
 
-    if (virAsprintf(pf_sysfs_device_link, NET_SYSFS "%s/%s",
-        ifname, file) < 0) {
-        virReportOOMError();
+    if (virAsprintf(pf_sysfs_device_link, NET_SYSFS "%s/%s", ifname, file) < 0)
         return -1;
-    }
-
     return 0;
 }
 
@@ -1070,12 +1084,9 @@ virNetDevSysfsDeviceFile(char **pf_sysfs_device_link, const char *ifname,
                      const char *file)
 {
 
-    if (virAsprintf(pf_sysfs_device_link, NET_SYSFS "%s/device/%s",
-        ifname, file) < 0) {
-        virReportOOMError();
+    if (virAsprintf(pf_sysfs_device_link, NET_SYSFS "%s/device/%s", ifname,
+                    file) < 0)
         return -1;
-    }
-
     return 0;
 }
 
@@ -1095,7 +1106,8 @@ virNetDevGetVirtualFunctions(const char *pfname,
                              virPCIDeviceAddressPtr **virt_fns,
                              unsigned int *n_vfname)
 {
-    int ret = -1, i;
+    int ret = -1;
+    size_t i;
     char *pf_sysfs_device_link = NULL;
     char *pci_sysfs_device_link = NULL;
     char *pciConfigAddr = NULL;
@@ -1110,10 +1122,8 @@ virNetDevGetVirtualFunctions(const char *pfname,
                                   n_vfname) < 0)
         goto cleanup;
 
-    if (VIR_ALLOC_N(*vfname, *n_vfname) < 0) {
-        virReportOOMError();
+    if (VIR_ALLOC_N(*vfname, *n_vfname) < 0)
         goto cleanup;
-    }
 
     for (i = 0; i < *n_vfname; i++)
     {
@@ -1661,16 +1671,12 @@ virNetDevReplaceVfConfig(const char *pflinkdev, int vf,
         goto cleanup;
 
     if (virAsprintf(&path, "%s/%s_vf%d",
-                    stateDir, pflinkdev, vf) < 0) {
-        virReportOOMError();
+                    stateDir, pflinkdev, vf) < 0)
         goto cleanup;
-    }
 
     if (virAsprintf(&fileData, "%s\n%d\n",
-                    virMacAddrFormat(&oldmac, macstr), oldvlanid) < 0) {
-        virReportOOMError();
+                    virMacAddrFormat(&oldmac, macstr), oldvlanid) < 0)
         goto cleanup;
-    }
     if (virFileWriteStr(path, fileData, O_CREAT|O_TRUNC|O_WRONLY) < 0) {
         virReportSystemError(errno, _("Unable to preserve mac/vlan tag "
                                       "for pf = %s, vf = %d"), pflinkdev, vf);
@@ -1699,10 +1705,8 @@ virNetDevRestoreVfConfig(const char *pflinkdev, int vf,
     int ifindex = -1;
 
     if (virAsprintf(&path, "%s/%s_vf%d",
-                    stateDir, pflinkdev, vf) < 0) {
-        virReportOOMError();
+                    stateDir, pflinkdev, vf) < 0)
         return rc;
-    }
 
     if (virFileReadAll(path, 128, &fileData) < 0) {
         goto cleanup;
