@@ -46,11 +46,15 @@
 # define in6_addr in6_addr_
 # define sockaddr_in6 sockaddr_in6_
 # define ipv6_mreq ipv6_mreq_
+# define in6addr_any in6addr_any_
+# define in6addr_loopback in6addr_loopback_
 # include <linux/in6.h>
 # include <linux/if_bridge.h> /* SYSFS_BRIDGE_ATTR  */
 # undef in6_addr
 # undef sockaddr_in6
 # undef ipv6_mreq
+# undef in6addr_any
+# undef in6addr_loopback
 
 # define JIFFIES_TO_MS(j) (((j)*1000)/HZ)
 # define MS_TO_JIFFIES(ms) (((ms)*HZ)/1000)
