@@ -126,10 +126,8 @@ vmwareConnectOpen(virConnectPtr conn,
         VIR_FREE(vmrun);
     }
 
-    if (VIR_ALLOC(driver) < 0) {
-        virReportOOMError();
+    if (VIR_ALLOC(driver) < 0)
         return VIR_DRV_OPEN_ERROR;
-    }
 
     if (virMutexInit(&driver->lock) < 0)
         goto cleanup;
@@ -254,8 +252,8 @@ vmwareStopVM(struct vmware_driver *driver,
              virDomainShutoffReason reason)
 {
     const char *cmd[] = {
-        VMRUN, "-T", PROGRAM_SENTINAL, "stop",
-        PROGRAM_SENTINAL, "soft", NULL
+        VMRUN, "-T", PROGRAM_SENTINEL, "stop",
+        PROGRAM_SENTINEL, "soft", NULL
     };
 
     vmwareSetSentinal(cmd, vmw_types[driver->type]);
@@ -275,8 +273,8 @@ static int
 vmwareStartVM(struct vmware_driver *driver, virDomainObjPtr vm)
 {
     const char *cmd[] = {
-        VMRUN, "-T", PROGRAM_SENTINAL, "start",
-        PROGRAM_SENTINAL, PROGRAM_SENTINAL, NULL
+        VMRUN, "-T", PROGRAM_SENTINEL, "start",
+        PROGRAM_SENTINEL, PROGRAM_SENTINEL, NULL
     };
     const char *vmxPath = ((vmwareDomainPtr) vm->privateData)->vmxPath;
 
@@ -448,8 +446,8 @@ vmwareDomainSuspend(virDomainPtr dom)
 
     virDomainObjPtr vm;
     const char *cmd[] = {
-      VMRUN, "-T", PROGRAM_SENTINAL, "pause",
-      PROGRAM_SENTINAL, NULL
+      VMRUN, "-T", PROGRAM_SENTINEL, "pause",
+      PROGRAM_SENTINEL, NULL
     };
     int ret = -1;
 
@@ -497,7 +495,7 @@ vmwareDomainResume(virDomainPtr dom)
 
     virDomainObjPtr vm;
     const char *cmd[] = {
-        VMRUN, "-T", PROGRAM_SENTINAL, "unpause", PROGRAM_SENTINAL,
+        VMRUN, "-T", PROGRAM_SENTINEL, "unpause", PROGRAM_SENTINEL,
         NULL
     };
     int ret = -1;
@@ -546,8 +544,8 @@ vmwareDomainReboot(virDomainPtr dom, unsigned int flags)
     const char * vmxPath = NULL;
     virDomainObjPtr vm;
     const char *cmd[] = {
-        VMRUN, "-T", PROGRAM_SENTINAL,
-        "reset", PROGRAM_SENTINAL, "soft", NULL
+        VMRUN, "-T", PROGRAM_SENTINEL,
+        "reset", PROGRAM_SENTINEL, "soft", NULL
     };
     int ret = -1;
 
@@ -987,7 +985,7 @@ vmwareConnectNumOfDefinedDomains(virConnectPtr conn)
 
     vmwareDriverLock(driver);
     vmwareDomainObjListUpdateAll(driver->domains, driver);
-    n = virDomainObjListNumOfDomains(driver->domains, 0);
+    n = virDomainObjListNumOfDomains(driver->domains, false, NULL, NULL);
     vmwareDriverUnlock(driver);
 
     return n;
@@ -1001,7 +999,7 @@ vmwareConnectNumOfDomains(virConnectPtr conn)
 
     vmwareDriverLock(driver);
     vmwareDomainObjListUpdateAll(driver->domains, driver);
-    n = virDomainObjListNumOfDomains(driver->domains, 1);
+    n = virDomainObjListNumOfDomains(driver->domains, true, NULL, NULL);
     vmwareDriverUnlock(driver);
 
     return n;
@@ -1016,7 +1014,7 @@ vmwareConnectListDomains(virConnectPtr conn, int *ids, int nids)
 
     vmwareDriverLock(driver);
     vmwareDomainObjListUpdateAll(driver->domains, driver);
-    n = virDomainObjListGetActiveIDs(driver->domains, ids, nids);
+    n = virDomainObjListGetActiveIDs(driver->domains, ids, nids, NULL, NULL);
     vmwareDriverUnlock(driver);
 
     return n;
@@ -1031,7 +1029,8 @@ vmwareConnectListDefinedDomains(virConnectPtr conn,
 
     vmwareDriverLock(driver);
     vmwareDomainObjListUpdateAll(driver->domains, driver);
-    n = virDomainObjListGetInactiveNames(driver->domains, names, nnames);
+    n = virDomainObjListGetInactiveNames(driver->domains, names, nnames,
+                                         NULL, NULL);
     vmwareDriverUnlock(driver);
     return n;
 }
@@ -1121,7 +1120,8 @@ vmwareConnectListAllDomains(virConnectPtr conn,
 
     vmwareDriverLock(driver);
     vmwareDomainObjListUpdateAll(driver->domains, driver);
-    ret = virDomainObjListExport(driver->domains, conn, domains, flags);
+    ret = virDomainObjListExport(driver->domains, conn, domains,
+                                 NULL, flags);
     vmwareDriverUnlock(driver);
     return ret;
 }
