@@ -75,6 +75,7 @@ struct data {
     const char *modelsName;
     unsigned int nmodels;
     const char *preferred;
+    unsigned int flags;
     int result;
 };
 
@@ -114,7 +115,7 @@ cpuTestLoadMultiXML(const char *arch,
     xmlNodePtr *nodes = NULL;
     virCPUDefPtr *cpus = NULL;
     int n;
-    int i;
+    size_t i;
 
     if (virAsprintf(&xml, "%s/cputestdata/%s-%s.xml", abs_srcdir, arch, name) < 0)
         goto cleanup;
@@ -261,7 +262,7 @@ cpuTestGuestData(const void *arg)
     virCPUDefPtr host = NULL;
     virCPUDefPtr cpu = NULL;
     virCPUDefPtr guest = NULL;
-    union cpuData *guestData = NULL;
+    virCPUDataPtr guestData = NULL;
     virCPUCompareResult cmpResult;
     virBuffer buf = VIR_BUFFER_INITIALIZER;
     char *result = NULL;
@@ -308,8 +309,7 @@ cpuTestGuestData(const void *arg)
 
 cleanup:
     VIR_FREE(result);
-    if (host)
-        cpuDataFree(host->arch, guestData);
+    cpuDataFree(guestData);
     virCPUDefFree(host);
     virCPUDefFree(cpu);
     virCPUDefFree(guest);
@@ -326,12 +326,12 @@ cpuTestBaseline(const void *arg)
     virCPUDefPtr baseline = NULL;
     unsigned int ncpus = 0;
     char *result = NULL;
-    unsigned int i;
+    size_t i;
 
     if (!(cpus = cpuTestLoadMultiXML(data->arch, data->name, &ncpus)))
         goto cleanup;
 
-    baseline = cpuBaseline(cpus, ncpus, NULL, 0);
+    baseline = cpuBaseline(cpus, ncpus, NULL, 0, data->flags);
     if (data->result < 0) {
         virResetLastError();
         if (!baseline)
@@ -359,7 +359,7 @@ cpuTestBaseline(const void *arg)
             cmp != VIR_CPU_COMPARE_IDENTICAL) {
             if (virTestGetVerbose()) {
                 fprintf(stderr,
-                        "\nbaseline CPU is incompatible with CPU %u\n", i);
+                        "\nbaseline CPU is incompatible with CPU %zu\n", i);
                 fprintf(stderr, "%74s", "... ");
             }
             ret = -1;
@@ -417,7 +417,7 @@ cpuTestHasFeature(const void *arg)
     const struct data *data = arg;
     int ret = -1;
     virCPUDefPtr host = NULL;
-    union cpuData *hostData = NULL;
+    virCPUDataPtr hostData = NULL;
     int result;
 
     if (!(host = cpuTestLoadXML(data->arch, data->host)))
@@ -427,7 +427,7 @@ cpuTestHasFeature(const void *arg)
                   NULL, NULL, NULL, NULL) < 0)
         goto cleanup;
 
-    result = cpuHasFeature(host->arch, hostData, data->name);
+    result = cpuHasFeature(hostData, data->name);
     if (data->result == -1)
         virResetLastError();
 
@@ -445,8 +445,7 @@ cpuTestHasFeature(const void *arg)
     ret = 0;
 
 cleanup:
-    if (host)
-        cpuDataFree(host->arch, hostData);
+    cpuDataFree(hostData);
     virCPUDefFree(host);
     return ret;
 }
@@ -512,12 +511,12 @@ mymain(void)
     }
 
 #define DO_TEST(arch, api, name, host, cpu,                             \
-                models, nmodels, preferred, result)                     \
+                models, nmodels, preferred, flags, result)              \
     do {                                                                \
         static struct data data = {                                     \
             arch, api, host, cpu, models,                               \
             models == NULL ? NULL : #models,                            \
-            nmodels, preferred, result    \
+            nmodels, preferred, flags, result                           \
         };                                                              \
         if (cpuTestRun(name, &data) < 0)                                \
             ret = -1;                                                   \
@@ -526,31 +525,31 @@ mymain(void)
 #define DO_TEST_COMPARE(arch, host, cpu, result)                        \
     DO_TEST(arch, API_COMPARE,                                          \
             host "/" cpu " (" #result ")",                              \
-            host, cpu, NULL, 0, NULL, result)
+            host, cpu, NULL, 0, NULL, 0, result)
 
 #define DO_TEST_UPDATE(arch, host, cpu, result)                         \
     do {                                                                \
         DO_TEST(arch, API_UPDATE,                                       \
                 cpu " on " host,                                        \
-                host, cpu, NULL, 0, NULL, 0);                           \
+                host, cpu, NULL, 0, NULL, 0, 0);                        \
         DO_TEST_COMPARE(arch, host, host "+" cpu, result);              \
     } while (0)
 
-#define DO_TEST_BASELINE(arch, name, result)                            \
+#define DO_TEST_BASELINE(arch, name, flags, result)                     \
     DO_TEST(arch, API_BASELINE, name, NULL, "baseline-" name,           \
-            NULL, 0, NULL, result)
+            NULL, 0, NULL, flags, result)
 
 #define DO_TEST_HASFEATURE(arch, host, feature, result)                 \
     DO_TEST(arch, API_HAS_FEATURE,                                      \
             host "/" feature " (" #result ")",                          \
-            host, feature, NULL, 0, NULL, result)
+            host, feature, NULL, 0, NULL, 0, result)
 
 #define DO_TEST_GUESTDATA(arch, host, cpu, models, preferred, result)   \
     DO_TEST(arch, API_GUEST_DATA,                                       \
             host "/" cpu " (" #models ", pref=" #preferred ")",         \
             host, cpu, models,                                          \
             models == NULL ? 0 : sizeof(models) / sizeof(char *),       \
-            preferred, result)
+            preferred, 0, result)
 
     /* host to host comparison */
     DO_TEST_COMPARE("x86", "host", "host", VIR_CPU_COMPARE_IDENTICAL);
@@ -583,6 +582,7 @@ mymain(void)
     DO_TEST_COMPARE("x86", "host", "pentium3-amd", VIR_CPU_COMPARE_INCOMPATIBLE);
     DO_TEST_COMPARE("x86", "host-amd", "pentium3-amd", VIR_CPU_COMPARE_SUPERSET);
     DO_TEST_COMPARE("x86", "host-worse", "nehalem-force", VIR_CPU_COMPARE_IDENTICAL);
+    DO_TEST_COMPARE("x86", "host-SandyBridge", "exact-force-Haswell", VIR_CPU_COMPARE_IDENTICAL);
 
     /* guest updates for migration
      * automatically compares host CPU with the result */
@@ -594,11 +594,12 @@ mymain(void)
     DO_TEST_UPDATE("x86", "host", "host-passthrough", VIR_CPU_COMPARE_IDENTICAL);
 
     /* computing baseline CPUs */
-    DO_TEST_BASELINE("x86", "incompatible-vendors", -1);
-    DO_TEST_BASELINE("x86", "no-vendor", 0);
-    DO_TEST_BASELINE("x86", "some-vendors", 0);
-    DO_TEST_BASELINE("x86", "1", 0);
-    DO_TEST_BASELINE("x86", "2", 0);
+    DO_TEST_BASELINE("x86", "incompatible-vendors", 0, -1);
+    DO_TEST_BASELINE("x86", "no-vendor", 0, 0);
+    DO_TEST_BASELINE("x86", "some-vendors", 0, 0);
+    DO_TEST_BASELINE("x86", "1", 0, 0);
+    DO_TEST_BASELINE("x86", "2", 0, 0);
+    DO_TEST_BASELINE("x86", "3", VIR_CONNECT_BASELINE_CPU_EXPAND_FEATURES, 0);
 
     /* CPU features */
     DO_TEST_HASFEATURE("x86", "host", "vmx", YES);
