@@ -18,6 +18,8 @@
 # include "testutilsqemu.h"
 # include "virstring.h"
 
+# define VIR_FROM_THIS VIR_FROM_NONE
+
 static virQEMUDriver driver;
 
 static int
@@ -28,6 +30,7 @@ testCompareXMLToXMLFiles(const char *inxml, const char *outxml, bool live)
     char *actual = NULL;
     int ret = -1;
     virDomainDefPtr def = NULL;
+    unsigned int flags = live ? 0 : VIR_DOMAIN_XML_INACTIVE;
 
     if (virtTestLoadFile(inxml, &inXmlData) < 0)
         goto fail;
@@ -35,11 +38,10 @@ testCompareXMLToXMLFiles(const char *inxml, const char *outxml, bool live)
         goto fail;
 
     if (!(def = virDomainDefParseString(inXmlData, driver.caps, driver.xmlopt,
-                                        QEMU_EXPECTED_VIRT_TYPES,
-                                        live ? 0 : VIR_DOMAIN_XML_INACTIVE)))
+                                        QEMU_EXPECTED_VIRT_TYPES, flags)))
         goto fail;
 
-    if (!(actual = virDomainDefFormat(def, VIR_DOMAIN_XML_SECURE)))
+    if (!(actual = virDomainDefFormat(def, VIR_DOMAIN_XML_SECURE | flags)))
         goto fail;
 
     if (STRNEQ(outXmlData, actual)) {
@@ -156,6 +158,7 @@ mymain(void)
     DO_TEST("eoi-enabled");
 
     DO_TEST("hyperv");
+    DO_TEST("hyperv-off");
 
     DO_TEST("hugepages");
     DO_TEST("nosharepages");
@@ -254,13 +257,16 @@ mymain(void)
 
     DO_TEST_FULL("seclabel-dynamic-baselabel", false, WHEN_INACTIVE);
     DO_TEST_FULL("seclabel-dynamic-override", false, WHEN_INACTIVE);
+    DO_TEST_FULL("seclabel-dynamic-labelskip", true, WHEN_INACTIVE);
     DO_TEST("seclabel-static");
+    DO_TEST_FULL("seclabel-static-labelskip", false, WHEN_ACTIVE);
     DO_TEST("seclabel-none");
     DO_TEST("numad-static-vcpu-no-numatune");
     DO_TEST("disk-scsi-lun-passthrough-sgio");
 
     DO_TEST("disk-scsi-disk-vpd");
     DO_TEST("disk-source-pool");
+    DO_TEST("disk-source-pool-mode");
 
     DO_TEST("disk-drive-discard");
 
@@ -287,8 +293,11 @@ mymain(void)
 
     DO_TEST("tpm-passthrough");
     DO_TEST("pci-bridge");
+    DO_TEST_DIFFERENT("pci-bridge-many-disks");
     DO_TEST_DIFFERENT("pci-autoadd-addr");
     DO_TEST_DIFFERENT("pci-autoadd-idx");
+    DO_TEST_DIFFERENT("pcie-root");
+    DO_TEST_DIFFERENT("q35");
 
     DO_TEST("hostdev-scsi-lsi");
     DO_TEST("hostdev-scsi-virtio-scsi");
@@ -302,6 +311,11 @@ mymain(void)
 
     DO_TEST_DIFFERENT("s390-defaultconsole");
 
+    DO_TEST("pcihole64");
+    DO_TEST_DIFFERENT("pcihole64-gib");
+    DO_TEST("pcihole64-none");
+    DO_TEST("pcihole64-q35");
+
     virObjectUnref(driver.caps);
     virObjectUnref(driver.xmlopt);
 
