@@ -66,6 +66,7 @@
 
 static unsigned int testDebug = -1;
 static unsigned int testVerbose = -1;
+static unsigned int testExpensive = -1;
 
 static unsigned int testOOM = 0;
 static size_t testCounter = 0;
@@ -581,6 +582,13 @@ virTestGetVerbose(void) {
     return testVerbose || virTestGetDebug();
 }
 
+unsigned int
+virTestGetExpensive(void) {
+    if (testExpensive == -1)
+        testExpensive = virTestGetFlag("VIR_TEST_EXPENSIVE");
+    return testExpensive;
+}
+
 int virtTestMain(int argc,
                  char **argv,
                  int (*func)(void))
