@@ -65,6 +65,7 @@ int virtTestDifferenceBin(FILE *stream,
 
 unsigned int virTestGetDebug(void);
 unsigned int virTestGetVerbose(void);
+unsigned int virTestGetExpensive(void);
 
 char *virtTestLogContentAndReset(void);
 
@@ -87,8 +88,9 @@ int virtTestMain(int argc,
                 perror(lib);                                            \
                 return EXIT_FAILURE;                                    \
             }                                                           \
-            if (virAsprintf(&newenv, "%s%s%s", preload ? preload : "",  \
-                            preload ? ":" : "", lib) < 0) {             \
+            if (!preload) {                                             \
+                newenv = (char *) lib;                                  \
+            } else if (virAsprintf(&newenv, "%s:%s", lib, preload) < 0) {   \
                 perror("virAsprintf");                                  \
                 return EXIT_FAILURE;                                    \
             }                                                           \
