@@ -172,9 +172,9 @@ typedef struct vshInterfaceList *vshInterfaceListPtr;
 static void
 vshInterfaceListFree(vshInterfaceListPtr list)
 {
-    int i;
+    size_t i;
 
-    if (list && list->nifaces) {
+    if (list && list->ifaces) {
         for (i = 0; i < list->nifaces; i++) {
             if (list->ifaces[i])
                 virInterfaceFree(list->ifaces[i]);
@@ -189,7 +189,7 @@ vshInterfaceListCollect(vshControl *ctl,
                         unsigned int flags)
 {
     vshInterfaceListPtr list = vshMalloc(ctl, sizeof(*list));
-    int i;
+    size_t i;
     int ret;
     char **activeNames = NULL;
     char **inactiveNames = NULL;
@@ -300,10 +300,10 @@ finished:
     success = true;
 
 cleanup:
-    for (i = 0; i < nActiveIfaces; i++)
+    for (i = 0; nActiveIfaces != -1 && i < nActiveIfaces; i++)
         VIR_FREE(activeNames[i]);
 
-    for (i = 0; i < nInactiveIfaces; i++)
+    for (i = 0; nInactiveIfaces != -1 && i < nInactiveIfaces; i++)
         VIR_FREE(inactiveNames[i]);
 
     VIR_FREE(activeNames);
@@ -349,7 +349,7 @@ cmdInterfaceList(vshControl *ctl, const vshCmd *cmd ATTRIBUTE_UNUSED)
     bool all = vshCommandOptBool(cmd, "all");
     unsigned int flags = VIR_CONNECT_LIST_INTERFACES_ACTIVE;
     vshInterfaceListPtr list = NULL;
-    int i;
+    size_t i;
 
     if (inactive)
         flags = VIR_CONNECT_LIST_INTERFACES_INACTIVE;
