@@ -421,9 +421,9 @@ typedef struct vshNetworkList *vshNetworkListPtr;
 static void
 vshNetworkListFree(vshNetworkListPtr list)
 {
-    int i;
+    size_t i;
 
-    if (list && list->nnets) {
+    if (list && list->nets) {
         for (i = 0; i < list->nnets; i++) {
             if (list->nets[i])
                 virNetworkFree(list->nets[i]);
@@ -438,7 +438,7 @@ vshNetworkListCollect(vshControl *ctl,
                       unsigned int flags)
 {
     vshNetworkListPtr list = vshMalloc(ctl, sizeof(*list));
-    int i;
+    size_t i;
     int ret;
     char **names = NULL;
     virNetworkPtr net;
@@ -658,7 +658,7 @@ static bool
 cmdNetworkList(vshControl *ctl, const vshCmd *cmd ATTRIBUTE_UNUSED)
 {
     vshNetworkListPtr list = NULL;
-    int i;
+    size_t i;
     bool inactive = vshCommandOptBool(cmd, "inactive");
     bool all = vshCommandOptBool(cmd, "all");
     bool persistent = vshCommandOptBool(cmd, "persistent");
@@ -918,7 +918,7 @@ cmdNetworkUpdate(vshControl *ctl, const vshCmd *cmd)
     const char *affected;
 
     if (!(network = vshCommandOptNetwork(ctl, cmd, NULL)))
-        goto cleanup;
+        return false;
 
     if (vshCommandOptStringReq(ctl, cmd, "command", &commandStr) < 0)
         goto cleanup;
